function moveElements(source, destination) {
    while (source.childNodes.length) destination.append(source.firstChild);
}

function show(element) {
    element.classList.add('show');
}

function hide(element) {
    element.classList.remove('show');
}

function disable(element) {
    element.disabled = true;
    element.classList.add('disabled');
}

function enable(element) {
    element.disabled = false;
    element.classList.remove('disabled');
}

function hilight(element) {
    element.classList.add('hilight');
}

function unhilight(element) {
    element.classList.remove('hilight');
}

function complete(element) {
    element._completed = true;
    if (typeof element._toReveal != "undefined") {
        for (var i = 0; i < element._toReveal.length; i++) {
            element._toReveal[i].classList.remove('hidden');
        }
    }
    if (typeof element._toHide != "undefined") {
        for (var i = 0; i < element._toHide.length; i++) {
            element._toHide[i].classList.add('hidden');
        }
    }
    sm.page.checkCompleted();
}

function uncomplete(element) {
    if (typeof element._toReveal != "undefined") {
        for (var i = 0; i < element._toReveal.length; i++) {
            element._toReveal[i].classList.add('hidden');
        }
    }
    if (typeof element._toHide != "undefined") {
        for (var i = 0; i < element._toHide.length; i++) {
            element._toHide[i].classList.remove('hidden');
        }
    }
}

function getElementsBySMName(element, names) {
    var elements = [];
    for (var i = 0; i < names.length; i++) {
        var name = names[i];
        var foundElement = undefined;
        if (name == 'next') {
            foundElement = element.nextElementSibling;
        } else if (name == 'me') {
            foundElement = element;
        } else {
            foundElement = sm.page.scanningElement.querySelector('[sm-name="' + name + '"]');
        }
        elements.push(foundElement);
    }
    return elements;
}

function toComplete(element) {
    element._completed = false;
    element.classList.add('sm-toComplete');

    if (element.hasAttribute('sm-reveal')) {
        element._toReveal = getElementsBySMName(element, element.getAttribute('sm-reveal').split(','));
        for (var i = 0; i < element._toReveal.length; i++) {
            element._toReveal[i].classList.add('hidden');
        }
    }
    if (element.hasAttribute('sm-hide')) {
        element._toHide = getElementsBySMName(element, element.getAttribute('sm-hide').split(','));
    }
}

function createSvgElement(tagName) {
    return document.createElementNS("http://www.w3.org/2000/svg", tagName);
}

function elem(name) {
    return sm.page.current._namedElements[name];
}

function style(name) {
    return sm.page.current._namedElements[name].style;
}

function hideElems(names) {
    var nameList = names.split(',');
    for (var i = 0; i < nameList.length; i++) {
        elem(nameList[i]).classList.add('hidden');
    }
}

function showElems(names) {
    var nameList = names.split(',');
    for (var i = 0; i < nameList.length; i++) {
        elem(nameList[i]).classList.remove('hidden');
    }
}



var sm = {

    user: {

        surname: "",
        name: "",
        grade: ""

    },

    createElement(name) {
        return document.createElement('sm-' + name);
    },

    capsuleContent(element, containerName) {
        var container = sm.createElement(containerName);
        sm.moveTo(element.childNodes, container);
        return container;
    },

    moveTo(elements, destination) {
        while (elements.length) {
            destination.append(elements[0]);
        }
    },



    page: {

        current: undefined,
        scanningElement: undefined,
        nextButtonTooltipShown: false,
        prevButtonTooltipShown: false,

        parse(element) {
            // Eintrag in der Navigation erstellen:
            element._sectionElement = element.parentElement;
            sm.navigation.createEntry(element);

            // Inhalte in einen Container packen:
            var contentElement = document.createElement('div');
            contentElement.classList.add('content');
            moveElements(element, contentElement);
            element.append(contentElement);

            // TITEL der Seite erstellen
            if (element.hasAttribute('sm-title')) {
                var titleContainer = document.createElement('div');
                titleContainer.classList.add('titlecontainer');

                var titleElement = document.createElement('div');
                titleElement.classList.add('title');
                titleElement.innerHTML = element.getAttribute('sm-title');
                titleContainer.append(titleElement);

                element.prepend(titleContainer);
            }

            // Methode abfragen, für Aktionen nach Verlassen dieser Seite:
            if (element.hasAttribute('sm-leave')) {
                element._leave = eval(element.getAttribute('sm-leave'));
            }

            // Alle Elemente sammeln, die einen Namen haben:
            var namedElements = element.querySelectorAll('[sm-name]');
            element._namedElements = {};
            for (var i = 0; i < namedElements.length; i++) {
                element._namedElements[namedElements[i].getAttribute('sm-name')] = namedElements[i];
            }

            element._completed = false;

            element._elementsToComplete = Array.from(element.getElementsByClassName('sm-toComplete'));
            //sm.mustFilled.parseFrom(element, element.getElementsByClassName('sm-mustfilled'));
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.page.parse(elements[i]);
            }
        },

        checkCompleted() {
            var element = sm.page.current;

            var completed = true;

            // Überprüfen ob alle zu bearbeitenden Elemente bearbeitet wurden:
            for (var i = 0; i < element._elementsToComplete.length; i++) {
                if (!element._elementsToComplete[i]._completed) {
                    completed = false;
                    break;
                }
            }

            if (element.getElementsByClassName('sm-uncomplete').length > 0) {
                completed = false;
            }

            // Wurde bis nach unten gescrollt?
            var container = document.getElementById('sm-contentcontainer');
            if (container.scrollHeight - container.clientHeight > 100 && container.scrollTop < container.scrollHeight - container.clientHeight - 100) {
                completed = false;
            }

            if (completed) {
                if (element._next != undefined) {
                    sm.button.enable(document.getElementById('sm-nextbutton'));
                    if (element._completed == false) {
                        sm.tooltip.show(document.getElementById('sm-nextbutton-tooltip'));
                    }
                } else {
                    sm.button.disable(document.getElementById('sm-nextbutton'));
                }
                element._completed = true;
            } else {
                element._completed = false;
                sm.button.disable(document.getElementById('sm-nextbutton'));
            }
        },

        select(element) {
            if (sm.page.current != undefined) {
                if (typeof sm.page.current._leave !== "undefined") {
                    sm.page.current._leave();
                }
                if (element._sectionElement != sm.page.current._sectionElement) {
                    sm.page.current._sectionElement.classList.remove('current');
                }
                sm.page.current.classList.remove('current');
            }

            // Neue Seite in der Navigation registieren:
            sm.navigation.moveTo(element);

            sm.page.current = element;
            element.classList.add('current');
            element._sectionElement.classList.add('current');

            // Neue aktuelle Seite automatisch nach oben scrollen:
            document.getElementById('sm-contentcontainer').scrollTop = 0;

            // Prüfen ob eine nächste Seite existiert und ob die aktuelle Seite fertig ist:
            sm.page.checkCompleted();

            // Alle Werte updaten:
            var onEnterElements = element.querySelectorAll('[sm-onEnter]');
            for (var i = 0; i < onEnterElements.length; i++) {
                var result = function(code){ return eval(code); }.call(onEnterElements[i], onEnterElements[i].getAttribute('sm-onEnter'));
            }
            sm.value.updateFrom(element.getElementsByClassName('sm-value'));

            // Prüfen ob eine vorherige Seite existiert:
            if (element._prev != undefined) {
                sm.button.enable(document.getElementById('sm-prevbutton'));
            } else {
                sm.button.disable(document.getElementById('sm-prevbutton'));
            }
            
            if (element.hasAttribute('sm-onEnter')) {
                setTimeout(function(){ eval(element.getAttribute('sm-onEnter')); }, 10);
            }
        },

        goToPrev() {
            if (sm.page.current._prev != undefined) {
                sm.page.select(sm.page.current._prev);
            }
        },

        goToNext() {
            if (sm.page.current._next != undefined) {
                sm.page.select(sm.page.current._next);
                sm.tooltip.hide(document.getElementById('sm-nextbutton-tooltip'));
            }
        }

    },



    button: {

        define(element, action) {
            element._action = action;
            element.addEventListener('click', sm.button.click);
        },

        click() {
            if (!this.classList.contains('disabled')) {
                this._action();
            }
        },

        enable(element) {
            element.classList.remove('disabled');
        },

        disable(element) {
            element.classList.add('disabled');
        }

    },



    box: {

        parse(element) {
            if (element.hasAttribute('sm-title')) {
                //var titleContainerElement = document.createElement('div');
            }

            var contentElement = document.createElement('div');
            contentElement.classList.add('content');
            moveElements(element, contentElement);

            if (typeof element._controlsElement != "undefined") {
                element.append(element._controlsElement);
            }

            element.append(contentElement);
            element.classList.add('sm-box');
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.box.parse(elements[i]);
            }
        },

        createControls(element, ontop = false) {
            var controlsElement = document.createElement('sm-controls');

            element._controlsElement = controlsElement;
            if (ontop) {
                element.prepend(controlsElement);
            } else {
                element.append(controlsElement);
            }
        }

    },



    inputBox: {

        parse(pageElement, element) {
            if (element.hasAttribute('sm-callback')) {
                element._callback = eval(element.getAttribute('sm-callback'));
            } else {
                element._callback = sm.inputBox.standardCallback;
            }

            if (element.hasAttribute('sm-saveIn')) {
                element._saveIn = eval(element.getAttribute('sm-saveIn'));
            } else {
                element._saveIn = {};
            }

            var inputs = element.getElementsByClassName('sm-input');
            element._inputs = [];
            for (var i = 0; i < inputs.length; i++) {
                element._inputs.push(inputs[i]);
            }

            sm.box.createControls(element);

            // Knopf zum Bestätigen der Eingaben erstellen:
            var submitElement = document.createElement('button');
            submitElement.innerHTML = 'Bestätigen';
            submitElement._inputBoxElement = element;
            submitElement.addEventListener('click', sm.inputBox.submit);

            element._submitElement = submitElement;
            element._controlsElement.append(submitElement);

            // Wurde das Attribut "sm-notEditable" angegeben?
            if (!element.hasAttribute('sm-notEditable')) {
                var reviseElement = document.createElement('button');
                reviseElement.innerHTML = 'Ändern';
                reviseElement._inputBoxElement = element;
                reviseElement.addEventListener('click', sm.inputBox.revise);
                disable(reviseElement);
                
                element._reviseElement = reviseElement;
                element._controlsElement.append(reviseElement);
            }

            // Gibt es Hilfestellungen, die eingeblendet werden sollen?
            if (element.hasAttribute('sm-cues')) {
                var cues = element.getAttribute('sm-cues').split(',');
                element._cues = [];
                for (var i = 0; i < cues.length; i++) {
                    var cueElement = sm.page.scanningElement.querySelector('[sm-name="' + cues[i] + '"]');
                    element._cues.push(cueElement);
                    cueElement.classList.add('sm-cue');
                    cueElement.classList.add('hidden');
                }

                var cueElement = document.createElement('button');
                cueElement.innerHTML = 'Hilfe einblenden';
                cueElement._inputBoxElement = element;
                cueElement.addEventListener('click', sm.inputBox.toggleCues);

                element._cueElement = cueElement;
                element._controlsElement.append(cueElement);
            }

            // Feedback-Elemente auffinden und an das Ende der Box stellen:
            var feedbacks = element.getElementsByTagName('sm-feedback');
            element._feedbacks = {};
            if (feedbacks.length > 0) {
                var feedbackContainerElement = document.createElement('div');
                feedbackContainerElement.classList.add('sm-feedbacks');
                while (feedbacks.length) {
                    element._feedbacks[feedbacks[0].getAttribute('sm-name')] = feedbacks[0];
                    feedbackContainerElement.append(feedbacks[0]);
                }
                element.append(feedbackContainerElement);
            }

            // Signalisieren, dass dieses Eingabeformular notwendig ist:
            toComplete(element);
        },

        parseFrom(pageElement, elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.inputBox.parse(pageElement, elements[i]);
            }
        },

        hideFeedbacks(element) {
            for (key in element._feedbacks) {
                hide(element._feedbacks[key]);
            }
        },

        unhilightInputs(element) {
            for (var i = 0; i < element._inputs.length; i++) {
                unhilight(element._inputs[i]);
            }
        },

        submit() {
            var inputBox = this._inputBoxElement;

            sm.inputBox.hideFeedbacks(inputBox);
            sm.inputBox.unhilightInputs(inputBox);
            inputBox._callback.submit(inputBox, inputBox._inputs, inputBox._feedbacks);

            if (inputBox.hasAttribute('sm-onSubmit')) {
                eval(inputBox.getAttribute('sm-onSubmit'));
            }
        },

        evaluate(element, rules) {
            var correct = true;
            for (key in rules.conditions) {
                if (!rules.conditions[key].condition()) {
                    show(element._feedbacks[rules.conditions[key].feedback]);
                    hilight(element._inputs[key]);
                    correct = false;
                }
            }
            if (correct) {
                show(element._feedbacks[rules.allCorrect.feedback]);
                for (var i = 0; i < element._inputs.length; i++) {
                    disable(element._inputs[i]);
                }
                sm.inputBox.setCompleted(element);
            }
        },

        revise() {
            var element = this._inputBoxElement;
            element._completed = false;
            enable(element._submitElement);
            disable(element._reviseElement);

            for (var i = 0; i < element._inputs.length; i++) {
                enable(element._inputs[i]);
            }
            for (key in element._feedbacks) {
                hide(element._feedbacks[key]);
            }
        },

        toggleCues() {
            var element = this._inputBoxElement;

            this.classList.toggle('toggled');
            if (this.classList.contains('toggled')) {
                this.innerHTML = 'Hilfe ausblenden';
                for (var i = 0; i < element._cues.length; i++) {
                    element._cues[i].classList.remove('hidden');
                    element._cues[i].style.backgroundColor = 'yellow';
                    element._cues[i].getBoundingClientRect();
                    element._cues[i].style.backgroundColor = 'transparent';
                }
                document.getElementById('sm-contentcontainer').scrollTop = element._cues[0].offsetTop;
            } else {
                this.innerHTML = 'Hilfe einblenden';
                for (var i = 0; i < element._cues.length; i++) {
                    element._cues[i].classList.add('hidden');
                }
            }
        },

        setCompleted(element) {
            complete(element)
            disable(element._submitElement);
            if (typeof element._reviseElement != "undefined") {
                enable(element._reviseElement);
            }

            for (var i = 0; i < element._inputs.length; i++) {
                disable(element._inputs[i]);
            }

            sm.value.updateFrom(element.getElementsByClassName('sm-value'));
        },

        standardCallback: {
            submit(element, inputs, feedbacks) {
                var correct = true;

                for (var i = 0; i < inputs.length; i++) {
                    var input = inputs[i];

                    var value = input._value(input);
                    element._saveIn[input.getAttribute('sm-name')] = value;

                    if (!sm.input.conditions[input.getAttribute('sm-condition')](input, value)) {
                        show(feedbacks[input.getAttribute('sm-feedback')]);
                        hilight(input);
                        correct = false;
                    }
                }

                if (correct) {
                    if (feedbacks.hasOwnProperty('correct')) {
                        show(feedbacks['correct']);
                    }
                    sm.inputBox.setCompleted(element);
                }
            }
        }

    },



    question: {
        parse(question) {
            var feedbacks = question.getElementsByTagName('sm-feedback');
            var feedbackContainer = sm.createElement('feedbackcontainer');
            sm.moveTo(feedbacks, feedbackContainer);

            var itemContainer = sm.capsuleContent(question, 'itemcontainer');
            question.append(itemContainer);
            
            question._items = Array.from(question.getElementsByTagName('SM-ITEM'));
            sm.question.item.parseFrom(question._items, question);

            controls = sm.createElement('controls');
            submit = document.createElement('button');
            submit.innerHTML = 'Bestätigen';
            submit._question = question;
            submit.addEventListener('click', sm.question.submit);
            controls.append(submit);
            question.append(controls);
            question.append(feedbackContainer);

            question.classList.add('sm-uncomplete');

            if (question.hasAttribute('sm-singlechoice')) {
                question._choice = undefined;
            }
        },

        parseFrom(questions) {
            for (var i = 0; i < questions.length; i++) {
                sm.question.parse(questions[i]);
            }
        },

        submit() {
            var question = this._question;

            var correct = true;
            for (var i = 0; i < question._items.length; i++) {
                if (!sm.question.item[question.getAttribute('sm-type')].eval(question._items[i])) {
                    correct = false;
                }
            }
            
            question.classList.remove('sm-correct')
            question.classList.remove('sm-wrong')

            if (correct) {
                question.classList.add('sm-correct');
                question.classList.remove('sm-uncomplete');
                sm.page.checkCompleted();
                this.disabled = true;

                if (question.hasAttribute('sm-onsubmit')) {
                    eval(question.getAttribute('sm-onsubmit'));
                }
            } else {
                question.classList.add('sm-wrong');
            }
        },



        item: {
            parse(item) {
                item.append(sm.capsuleContent(item, 'text'));

                index = sm.createElement('index');
                index.innerHTML = String.fromCharCode(97 + item._index) + ')';
                item.prepend(index);

                sm.question.item[item._question.getAttribute('sm-type')].parse(item);
            },

            parseFrom(items, question) {
                for (var i = 0; i < items.length; i++) {
                    items[i]._question = question;
                    items[i]._index = i;
                    sm.question.item.parse(items[i]);
                }
            },

            check: {
                parse(item) {
                    item.addEventListener('click', sm.question.item.check.click);

                    var checkbox = sm.createElement('checkbox');
                    item.prepend(checkbox);
                },

                click() {
                    if (this._question.classList.contains('sm-uncomplete')) {
                        if (this._question.hasAttribute('sm-singlechoice')) {
                            if (this._question._choice != undefined) {
                                this._question._choice.classList.remove('checked');
                            }
                            this._question._choice = this;
                        }
                        this.classList.add('checked');
                    }
                },

                eval(item) {
                    console.log(item.classList.contains('checked'));
                    console.log(item.hasAttribute('sm-correct'));
                    return (item.classList.contains('checked') == item.hasAttribute('sm-correct'));
                }
            },

            truefalse: {
                parse(item) {
                    var trueItem = sm.createElement('true');
                    var falseItem = sm.createElement('false');
                    trueItem.innerHTML = 'Richtig';
                    falseItem.innerHTML = 'Falsch';

                    var truefalse = sm.createElement('truefalse');

                    trueItem._item = item;
                    trueItem._container = truefalse;
                    falseItem._item = item;
                    falseItem._container = truefalse;
                    trueItem.addEventListener('click', sm.question.item.truefalse.clickTrue);
                    falseItem.addEventListener('click', sm.question.item.truefalse.clickFalse);

                    item._true = trueItem;
                    item._false = falseItem;
                    item._truefalse = truefalse;

                    truefalse.append(trueItem);
                    truefalse.append(falseItem);
                    item.append(truefalse);
                },

                clickTrue() {
                    if (this._item._question.classList.contains('sm-uncomplete')) {
                        this._container.classList.add('used');
                        this._container.classList.add('true');
                        this._container.classList.remove('false');
                    }
                },

                clickFalse() {
                    if (this._item._question.classList.contains('sm-uncomplete')) {
                        this._container.classList.add('used');
                        this._container.classList.add('false');
                        this._container.classList.remove('true');
                    }
                },

                eval(item) {
                    return (item._truefalse.classList.contains('used') && item._truefalse.classList.contains('true') == item.hasAttribute('sm-correct'));
                }
            }
        }
    },



    hilight: {

        parse(page, element) {
            var hilights = element.getAttribute('sm-hilight').split(',');
            element._hilightElements = [];
            for (var i = 0; i < hilights.length; i++) {
                var hilightElement = page.querySelector('[sm-name="' + hilights[i] + '"]');
                hilightElement.classList.add('hidden');
                element._hilightElements.push(hilightElement);
            }

            element.addEventListener('mouseenter', sm.hilight.show);
            element.addEventListener('mouseleave', sm.hilight.hide);
        },

        parseFrom(page, elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.hilight.parse(page, elements[i]);
            }
        },

        show() {
            for (var i = 0; i < this._hilightElements.length; i++) {
                this._hilightElements[i].classList.remove('hidden');
            }
        },

        hide() {
            for (var i = 0; i < this._hilightElements.length; i++) {
                this._hilightElements[i].classList.add('hidden');
            }
        }

    },



    value: {

        parse(element) {
            if (element.hasAttribute('sm-loadFrom')) {
                element._loadFrom = eval(element.getAttribute('sm-loadFrom'));
                element._name = element.getAttribute('sm-name');
                element._script = sm.value.loadFromScript;
            } else if (element.hasAttribute('sm-expression')) {
                var command = '(function(element) { return ' + element.getAttribute('sm-expression') + '; })';
                element._script = eval(command);
            }
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.value.parse(elements[i]);
            }
        },

        update(element) {
            element.innerHTML = element._script(element);
        },

        updateFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.value.update(elements[i]);
            }
        },

        loadFromScript(element) {
            var value = element._loadFrom[element._name]
            if (typeof value == "number") {
                return value.toLocaleString('de');
            } else {
                return value;
            }
        }

    },



    input: {

        parse(element) {
            switch (element.tagName) {
                case 'INPUT':
                    switch (element.type) {
                        case 'number':
                            element._value = sm.input.values.number;
                            break;
                        case 'text':
                            element._value = sm.input.values.text;
                            break;
                    }
                    break;
                case 'SELECT':
                    element._value = sm.input.values.text;
                case 'TEXTAREA':
                    element._value = sm.input.values.text;
                    break;
                case 'SM-OPTIONS':
                    element._value = sm.input.values.option;
                    break;
            }

            if (element.hasAttribute('sm-condition')) {
                element._condition = sm.input.conditions[element.getAttribute('sm-condition')];
            }
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.input.parse(elements[i]);
            }
        },

        values: {
            number(element) {
                return parseFloat(element.value);
            },

            text(element) {
                return element.value;
            },

            option(element) {
                if (element._current != undefined) {
                    if (element._current.hasAttribute('sm-text')) {
                        return element._current.getAttribute('sm-text');
                    } else {
                        return element._current.innerHTML;
                    }
                } else {
                    return '';
                }
            }
        },

        conditions: {
            inRange(element, value) {
                return (!isNaN(value) && value >= parseFloat(element.getAttribute('sm-conditionMin')) && value <= parseFloat(element.getAttribute('sm-conditionMax')));
            },

            validNumber(element, value) {
                return (!isNaN(value));
            },

            notEmpty(element, value) {
                return (value != "");
            },

            sentences(element, value) {
                return (value.split(' ').length > 8);
            },

            equal(element, value) {
                return (value == element.getAttribute('sm-conditionValue'));
            }
        },

    },



    commandBox: {

        parse(element) {
            // Sollen Element aufgedeckt werden, falls diese Anweisung ausgeführt wurde?
            if (element.hasAttribute('sm-reveal')) {
                // Button erstellen mit "Nächster Schritt":
                sm.box.createControls(element);

                var submitElement = document.createElement('button');
                submitElement.innerHTML = 'Nächster Schritt';
                submitElement._commandBoxElement = element;
                submitElement.addEventListener('click', sm.commandBox.submit);

                element._submitElement = submitElement;
                element._controlsElement.append(submitElement);

                toComplete(element);
            }
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.commandBox.parse(elements[i]);
            }
        },

        submit() {
            var commandBox = this._commandBoxElement;
            disable(this);

            if (commandBox.hasAttribute('sm-onSubmit')) {
                eval(commandBox.getAttribute('sm-onSubmit'));
            }

            complete(commandBox);
        }

    },



    stepBox: {

        parse(element) {
            // Sollen Elemente aufgedeckt werden, falls dieser Abschnitt fortgesetzt wird?
            sm.box.createControls(element, !element.hasAttribute('sm-growing') || element.hasAttribute('sm-top'));

            var goNextElement = document.createElement('button');
            goNextElement.innerHTML = 'Weiter';
            goNextElement._stepBoxElement = element;
            goNextElement.addEventListener('click', sm.stepBox.goNext);

            var goBackElement = document.createElement('button');
            goBackElement.innerHTML = 'Zurück';
            goBackElement._stepBoxElement = element;
            goBackElement.addEventListener('click', sm.stepBox.goBack);
            disable(goBackElement);

            element._goNextElement = goNextElement;
            element._goBackElement = goBackElement;
            element._controlsElement.append(goBackElement);
            element._controlsElement.append(goNextElement);

            element._stepElements = element.getElementsByTagName('sm-step');
            for (var i = 1; i < element._stepElements.length; i++) {
                toComplete(element._stepElements[i]);
                element._stepElements[i].classList.add('hidden');
            }
            element._step = 0;
            element._stepElements[0].classList.add('current');

            toComplete(element);
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.stepBox.parse(elements[i]);
            }
        },

        goNext() {
            element = this._stepBoxElement;
            
            element._step = element._step + 1;
            var stepElement = element._stepElements[element._step];
            var prevStepElement = element._stepElements[element._step - 1];

            prevStepElement.classList.remove('current');
            stepElement.classList.add('current');

            if (!element.hasAttribute('sm-growing')) {
                element._stepElements[element._step - 1].classList.add('hidden');
            }
            element._stepElements[element._step].classList.remove('hidden');
            complete(stepElement);

            if (prevStepElement.hasAttribute('sm-onLeave')) {
                eval(prevStepElement.getAttribute('sm-onLeave'));
            }
            if (stepElement.hasAttribute('sm-onDisplay')) {
                eval(stepElement.getAttribute('sm-onDisplay'));
            }

            if (element._step == element._stepElements.length - 1) {
                disable(this);
                complete(element);
            }

            enable(element._goBackElement);
        },

        goBack() {
            element = this._stepBoxElement;

            var prevStepElement = element._stepElements[element._step];
            var stepElement = element._stepElements[element._step - 1];

            stepElement.classList.add('current');
            prevStepElement.classList.remove('current');

            element._step = element._step - 1;
            prevStepElement.classList.add('hidden');
            stepElement.classList.remove('hidden');
            uncomplete(prevStepElement);

            if (prevStepElement.hasAttribute('sm-onLeave')) {
                eval(prevStepElement.getAttribute('sm-onLeave'));
            }
            if (stepElement.hasAttribute('sm-onDisplay')) {
                eval(stepElement.getAttribute('sm-onDisplay'));
            }

            if (element._step == 0) {
                disable(this);
            }

            enable(element._goNextElement);
        }

    },



    section: {

        parse(element) {
            if (!element.hasAttribute('sm-hideTitle')) {
                var pageElement = document.createElement('sm-page');
                pageElement.classList.add('sectiontitle');
                pageElement.setAttribute('sm-title', element.getAttribute('sm-title'));

                element.prepend(pageElement);
            }

            sm.navigation.createSection(element);
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.section.parse(elements[i]);
            }
        }

    },



    timer: {
        parse(timer) {
            timer._digits = document.createElement('sm-timer-digits');
            timer.append(timer._digits);

            timer._interval = undefined;
            timer._seconds = 0;
            timer._startSeconds = 0;

            if (timer.hasAttribute('sm-clickable')) {
                timer.addEventListener('click', sm.timer.onclick);
            }

            if (timer.hasAttribute('sm-seconds')) {
                sm.timer.setStartSeconds(timer, parseInt(timer.getAttribute('sm-seconds')));
            } else {
                sm.timer.setStartSeconds(timer, 60);
            }
        },

        parseFrom(timers) {
            for (var i = 0; i < timers.length; i++) {
                sm.timer.parse(timers[i]);
            }
        },

        setStartSeconds(timer, seconds) {
            timer._startSeconds = seconds;
            if (timer._interval == undefined) {
                sm.timer.setSeconds(timer, seconds);
            }
        },

        setSeconds(timer, seconds) {
            timer._seconds = seconds;
            timer._digits.innerHTML = seconds;
            if (sm.page.current != undefined && sm.page.current.hasAttribute('sm-onTimerTick')) {
                eval(sm.page.current.getAttribute('sm-onTimerTick'));
            }
        },

        start(timer) {
            if (timer._interval != undefined) {
                sm.timer.stop(timer);
            } else {
                timer._interval = setInterval(function(){ sm.timer.tick(timer); }, 1000);
                sm.timer.setSeconds(timer, timer._startSeconds);
                timer.classList.add('ticking');
            }
        },

        onclick() {
            sm.timer.start(this);
        },
    
        stop(timer) {
            clearInterval(timer._interval);
            timer._interval = undefined;
            timer.classList.remove('ticking');
            sm.timer.setSeconds(timer, timer._startSeconds);
        },
    
        tick(timer) {
            sm.timer.setSeconds(timer, timer._seconds - 1);
            if (timer._seconds == 0) {
                if (sm.page.current.hasAttribute('sm-onTimerFinished')) {
                    eval(sm.page.current.getAttribute('sm-onTimerFinished'));
                }
                sm.timer.stop(timer);
            }
        },
    },



    navigation: {
        
        sidebarElement: undefined,
        navigationElement: undefined,
        constructingSection: undefined,
        entryElements: [],

        init() {
            sm.navigation.sidebarElement = document.getElementById('sm-sidebar');
            sm.navigation.navigationElement = document.getElementById('sm-navigation')
        },

        toggle() {
            sm.navigation.sidebarElement.classList.toggle('retracted');
        },

        moveTo(element) {
            if (sm.page.current != undefined) {
                sm.page.current._entryElement.classList.remove('current');
            }

            element._entryElement.classList.remove('disabled');
            element._entryElement.classList.add('current');
        },

        createSection(element) {
            var sectionElement = document.createElement('sm-section');
            sectionElement._sectionElement = element;

            var titleElement = document.createElement('sm-title');
            titleElement.innerHTML = element.getAttribute('sm-title');
            sectionElement._titleElement = titleElement;

            var contentElement = document.createElement('sm-content');
            sectionElement._contentElement = contentElement;

            sectionElement.append(titleElement);
            sectionElement.append(contentElement);

            element._navigationElement = sectionElement;
            sm.navigation.navigationElement.append(sectionElement);
        },

        createEntry(element) {
            var entryElement = document.createElement('sm-entry');
            entryElement.classList.add('item');
            entryElement.classList.add('disabled');
            entryElement._pageElement = element;
            entryElement.innerHTML = element.getAttribute('sm-title');
            element._entryElement = entryElement;

            entryElement.addEventListener('click', sm.navigation.itemClick);

            element._sectionElement._navigationElement._contentElement.append(entryElement);
            sm.navigation.entryElements.push(entryElement);
        },

        itemClick() {
            if (!this.classList.contains('disabled')) {
                sm.page.select(this._pageElement);
            }
        },

        enableAllItems() {
            for (var i = 0; i < sm.navigation.entryElements.length; i++) {
                sm.navigation.entryElements[i].classList.remove('disabled');
            }
        },

    },

    /*
    navigation: {

        containerElement: undefined,
        constructingSection: undefined,
        itemElements: [],

        init() {
            sm.navigation.containerElement = document.getElementById('sm-navigationContainer');
        },

        toggle() {
            this.classList.toggle('toggled');
            document.getElementById('sm-footer').classList.toggle('retracted');
        },

        moveTo(element) {
            if (sm.page.current != undefined) {
                sm.page.current._itemElement.classList.remove('current');
            }

            element._itemElement.classList.remove('disabled');
            element._itemElement.classList.add('current');
        },

        createSection(element) {
            var sectionElement = document.createElement('div');
            sectionElement.classList.add('section');
            sectionElement._sectionElement = element;

            var titleElement = document.createElement('div');
            titleElement.classList.add('title');
            titleElement.innerHTML = element.getAttribute('sm-title');
            sectionElement._titleElement = titleElement;

            var contentElement = document.createElement('div');
            contentElement.classList.add('content');
            sectionElement._contentElement = contentElement;

            sectionElement.append(titleElement);
            sectionElement.append(contentElement);

            element._navigationElement = sectionElement;
            sm.navigation.containerElement.append(sectionElement);
        },

        createItem(element) {
            var itemElement = document.createElement('div');
            itemElement.classList.add('item');
            itemElement.classList.add('disabled');
            itemElement._pageElement = element;
            element._itemElement = itemElement;

            itemElement.addEventListener('click', sm.navigation.itemClick);

            element._sectionElement._navigationElement._contentElement.append(itemElement);
            sm.navigation.itemElements.push(itemElement);
        },

        itemClick() {
            if (!this.classList.contains('disabled')) {
                sm.page.select(this._pageElement);
            }
        },

        enableAllItems() {
            for (var i = 0; i < sm.navigation.itemElements.length; i++) {
                sm.navigation.itemElements[i].classList.remove('disabled');
            }
        }

    },*/


    modalImage: {
        parse(image) {
            image.addEventListener('click', sm.modalImage.modalClick)
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.modalImage.parse(elements[i]);
            }
        },

        modalClick() {
            document.getElementById('sm-modal-image').src = this.src;
            document.getElementById('sm-modal').classList.remove('hidden');
        }
    },



    tooltip: {

        parse(element) {
            element._element = document.getElementById(element.getAttribute('sm-element'));
            element.classList.add('hidden');
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.tooltip.parse(elements[i]);
            }
        },
        
        show(element) {
            element.classList.remove('hidden');
        },

        hide(element) {
            element.classList.add('hidden');
        }

    },



    options: {
        parse(element) {
            element._options = element.getElementsByTagName('sm-option');
            element._current = element.querySelector('[sm-selected]');
            if (element._current != undefined) {
                element._current.classList.add('current');
            } else {
                toComplete(element);
            }

            for (var i = 0; i < element._options.length; i++) {
                element._options[i]._parent = element;
                element._options[i].addEventListener('click', sm.options.click);
            }
        },

        parseFrom(elements) {
            for (var i = 0; i < elements.length; i++) {
                sm.options.parse(elements[i]);
            }
        },

        enable(option) {
            var parent = option._parent;

            if (!parent.classList.contains('disabled')) {
                if (parent._current != option) {
                    if (parent._current != undefined) {
                        sm.options.disable(parent._current);
                    } else {
                        complete(parent);
                    }
                    parent._current = option;
                    option.classList.add('current');

                    if (option.hasAttribute('sm-onClick')) {
                        eval(option.getAttribute('sm-onClick'));
                    }
                }
            }
        },

        disable(option) {
            var parent = option._parent;

            option.classList.remove('current');
        },

        click() {
            sm.options.enable(this);
        }
    },



    linkElements(elements) {
        for (var i = 0; i < elements.length; i++) {
            if (i > 0) {
                elements[i]._prev = elements[i - 1];
            } else {
                elements[i]._prev = undefined;
            }
            if (i < elements.length - 1) {
                elements[i]._next = elements[i + 1];
            } else {
                elements[i]._next = undefined;
            }
        }
    },

    scroll() {
        if (this.scrollTop >= this.scrollHeight - this.clientHeight - 100) {
            sm.page.checkCompleted();
        }
    }
};



window.onload = function () {
    sm.navigation.init();

    // Knöpfe und ihre Funktionen festlegen:
    sm.button.define(document.getElementById('sm-prevbutton'), sm.page.goToPrev);
    sm.button.define(document.getElementById('sm-nextbutton'), sm.page.goToNext);
    /*sm.button.define(document.getElementById('sm-toggleNavigationButton'), sm.navigation.toggle)*/

    sm.value.parseFrom(document.getElementsByClassName('sm-value'));
    sm.options.parseFrom(document.getElementsByTagName('sm-options'));
    sm.input.parseFrom(document.getElementsByClassName('sm-input'));
    sm.box.parseFrom(document.querySelectorAll('sm-box,sm-inputBox,sm-commandBox,sm-stepBox'));
    sm.section.parseFrom(document.getElementsByTagName('section'));
    sm.tooltip.parseFrom(document.getElementsByTagName('sm-tooltip'));
    sm.plot.parseFrom(document.getElementsByTagName('sm-plot'));
    sm.question.parseFrom(document.getElementsByTagName('sm-question'));
    sm.modalImage.parseFrom(document.querySelectorAll('img[sm-modal]'));
    sm.timer.parseFrom(document.getElementsByTagName('sm-timer'));

    var pageElements = document.getElementsByTagName('sm-page');
    for (var i = 0; i < pageElements.length; i++) {
        var pageElement = pageElements[i];
        sm.page.scanningElement = pageElement;

        sm.hilight.parseFrom(pageElement, pageElement.getElementsByTagName('sm-hilight'));
        sm.inputBox.parseFrom(pageElement, pageElement.getElementsByTagName('sm-inputBox'));
        sm.commandBox.parseFrom(pageElement.getElementsByTagName('sm-commandBox'));
        sm.stepBox.parseFrom(pageElement.getElementsByTagName('sm-stepBox'));
    }
    sm.page.parseFrom(pageElements);

    sm.linkElements(pageElements);

    var debugElements = document.getElementsByClassName('debug');
    if (debugElements.length) {
        sm.navigation.enableAllItems();
        sm.page.select(debugElements[0]);
    } else {
        sm.page.select(pageElements[0]);
    }

    var container = document.getElementById('sm-contentcontainer');
    container.addEventListener('scroll', sm.scroll);

    document.body.classList.remove('loading');
}